'use strict'

const checkWarnings = require('./check-warning-header')
const fetch = require('make-fetch-happen')
const registryKey = require('./registry-key')
const url = require('url')

module.exports = regFetch
function regFetch (uri, registry, opts) {
  const startTime = Date.now()
  return fetch(uri, {
    agent: opts.agent,
    algorithms: opts.algorithms,
    cache: getCacheMode(opts),
    cacheManager: opts.cache,
    headers: getHeaders(uri, registry, opts),
    integrity: opts.integrity,
    proxy: opts.proxy,
    referer: opts.refer,
    retry: opts.retry,
    timeout: opts.timeout
  }).then(res => {
    if (res.headers.has('npm-notice') && !res.headers.has('x-local-cache')) {
      opts.log.warn('notice', res.headers.get('npm-notice'))
    }
    checkWarnings(res, registry, opts)
    res.body.on('end', () => {
      const elapsedTime = Date.now() - startTime
      const attempt = res.headers.get('x-fetch-attempts')
      const attemptStr = attempt && attempt > 1 ? ` attempt #${attempt}` : ''
      const cacheStr = res.headers.get('x-local-cache') ? ' (from cache)' : ''
      opts.log.http(
        'fetch',
        `GET ${res.status} ${uri} ${elapsedTime}ms${attemptStr}${cacheStr}`
      )
    })
    if (res.status >= 400) {
      const err = new Error(`${res.statusText}: ${uri}`)
      err.code = `E${res.status}`
      err.response = res
      throw err
    } else {
      return res
    }
  })
}

function getCacheMode (opts) {
  return opts.offline
  ? 'only-if-cached'
  : opts.preferOffline
  ? 'force-cache'
  : opts.preferOnline
  ? 'no-cache'
  : 'default'
}

function getHeaders (uri, registry, opts) {
  const headers = Object.assign({
    'npm-in-ci': opts.isFromCI,
    'npm-scope': opts.projectScope,
    'user-agent': opts.userAgent,
    'referer': opts.refer
  }, opts.headers)
  const auth = (
    opts.auth &&
    // If these two are on different hosts, don't send credentials.
    // This is mainly used by the tarball fetcher.
    url.parse(uri).host === url.parse(registry).host &&
    opts.auth[registryKey(registry)]
  )
  if (auth && auth.token) {
    headers.authorization = `Bearer ${auth.token}`
  }
  return headers
}
